﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Managers;

namespace AdvQuestDev
{
    public partial class ui : Form
    {
        public ui()
        {
            InitializeComponent();
            propertyGrid1.PropertySort = PropertySort.NoSort;
            Refresh();
        }

        #region On Load

        private void gui_load(object sender, System.EventArgs e)
        {
            // Set Tooltips 
            System.Windows.Forms.ToolTip toolTip = new System.Windows.Forms.ToolTip();
            toolTip.AutoPopDelay = 5000;
            toolTip.InitialDelay = 1000;
            toolTip.ReshowDelay = 500;
            toolTip.ShowAlways = true;
            toolTip.SetToolTip(this.lbBag1, "Inventory Bag 1. Highlight required Item (Ctrl+Click for multiple). Double click to remove");
            toolTip.SetToolTip(this.lbBag2, "Inventory Bag 2. Highlight required Item (Ctrl+Click for multiple). Double click to remove");
            toolTip.SetToolTip(this.lbBag3, "Inventory Bag 3. Highlight required Item (Ctrl+Click for multiple). Double click to remove");
            toolTip.SetToolTip(this.lbBag4, "Inventory Bag 4. Highlight required Item (Ctrl+Click for multiple). Double click to remove");
            toolTip.SetToolTip(this.lbKeyItems, "Key Items. Highlight required Item. Click blank space to remove");
            toolTip.SetToolTip(this.button1, "Refreshes the data. Use after new quests are recieved and/or quest objectives have been updated");
            toolTip.SetToolTip(this.button2, "PickUp selected Quest from current target");
            toolTip.SetToolTip(this.button3, "TurnIn selected Quest and/or Item/s to current target");
            toolTip.SetToolTip(this.button4, "If Condition = current highlighted parameters");
            toolTip.SetToolTip(this.button5, "TalkTo current target");
            toolTip.SetToolTip(this.button6, "UseTransport at current target");
            toolTip.SetToolTip(this.button7, "HandOver selected Item/s to current Target");
            toolTip.SetToolTip(this.button8, "NoCombatMoveTo current targets location. Target yourself if you want your current location");
            toolTip.SetToolTip(this.button9, "TeleportTo current targeted Aetheryte");
            toolTip.SetToolTip(this.button10, "Use current selected object (Singular)");
            toolTip.SetToolTip(this.button11, "Use selected item on current target (Singular)");
            toolTip.SetToolTip(this.button12, "Pickup selected Daily Quest from current target");

            // Set Multiple Select
            lbBag1.SelectionMode = SelectionMode.MultiExtended;
            lbBag2.SelectionMode = SelectionMode.MultiExtended;
            lbBag3.SelectionMode = SelectionMode.MultiExtended;
            lbBag4.SelectionMode = SelectionMode.MultiExtended;
        }

        #endregion OnLoad

        #region Data Sources & Event Args

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            propertyGrid1.SelectedObject = comboBox1.SelectedItem;
        }

        private void Refresh()
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var questData = comboBox1.SelectedItem as QuestWork;
                var quests = QuestLogManager.Quests();
                comboBox1.DataSource = quests;
                comboBox1.DisplayMember = "Name";
                comboBox1.ValueMember = "Name";
                if (questData != null)
                {
                    if (quests.Length > 0)
                        comboBox1.SelectedItem = quests.FirstOrDefault(r => r.Name == questData.Name) ?? quests[0];
                }
                lbKeyItems.DataSource = InventoryManager.GetBagByInventoryBagId(InventoryBagId.KeyItems).FilledSlots;
                lbKeyItems.DisplayMember = "Name";
                lbKeyItems.ValueMember = "Name";
                lbBag1.DataSource = InventoryManager.GetBagByInventoryBagId(InventoryBagId.Bag1).FilledSlots;
                lbBag1.DisplayMember = "Name";
                lbBag1.ValueMember = "Name";
                lbBag2.DataSource = InventoryManager.GetBagByInventoryBagId(InventoryBagId.Bag2).FilledSlots;
                lbBag2.DisplayMember = "Name";
                lbBag2.ValueMember = "Name";
                lbBag3.DataSource = InventoryManager.GetBagByInventoryBagId(InventoryBagId.Bag3).FilledSlots;
                lbBag3.DisplayMember = "Name";
                lbBag3.ValueMember = "Name";
                lbBag4.DataSource = InventoryManager.GetBagByInventoryBagId(InventoryBagId.Bag4).FilledSlots;
                lbBag4.DisplayMember = "Name";
                lbBag4.ValueMember = "Name";

            }
        }

        public class QuestWrapper
        {
            public QuestWrapper(QuestWork value)
            {
                Value = value;
            }
            public QuestWork Value { get; set; }

            public override string ToString()
            {
                return Value.Name;
            }
        }

        private void lbKeyItems_MouseDown(object sender, MouseEventArgs e)
        {
            var index = lbKeyItems.IndexFromPoint(e.Location);
            if (index == ListBox.NoMatches)
            {
                lbKeyItems.ClearSelected();
            }
        }

        private void lbBag1_MouseDown(object sender, MouseEventArgs e)
        {
            var index = lbBag1.IndexFromPoint(e.Location);
            if (index == ListBox.NoMatches)
            {
                lbBag1.ClearSelected();
            }
        }

        private void lbBag1_DoubleClick(object sender, MouseEventArgs e)
        {
            lbBag1.ClearSelected();
        }

        private void lbBag2_MouseDown(object sender, MouseEventArgs e)
        {
            var index = lbBag2.IndexFromPoint(e.Location);
            if (index == ListBox.NoMatches)
            {
                lbBag1.ClearSelected();
            }
        }

        private void lbBag2_DoubleClick(object sender, MouseEventArgs e)
        {
            lbBag2.ClearSelected();
        }


        private void lbBag3_MouseDown(object sender, MouseEventArgs e)
        {
            var index = lbBag3.IndexFromPoint(e.Location);
            if (index == ListBox.NoMatches)
            {
                lbBag1.ClearSelected();
            }
        }

        private void lbBag3_DoubleClick(object sender, MouseEventArgs e)
        {
            lbBag3.ClearSelected();
        }


        private void lbBag4_MouseDown(object sender, MouseEventArgs e)
        {
            var index = lbBag4.IndexFromPoint(e.Location);
            if (index == ListBox.NoMatches)
            {
                lbBag1.ClearSelected();
            }
        }

        private void lbBag4_DoubleClick(object sender, MouseEventArgs e)
        {
            lbBag4.ClearSelected();
        }

        #endregion Data Sources & Event Args

        #region Refresh

        private void button1_Click(object sender, EventArgs e)
        {
            Refresh();
        }

        #endregion Refresh

        #region PickupQuest

        private void button2_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var questData = comboBox1.SelectedItem as QuestWork;
                var location = Core.Target.Location.ToString().Remove(0, 1);
                location = location.Remove(location.Length - 1, 1);
                if (questData != null)
                {
                    var output = string.Format(pickupQuest, questData.Name, questData.GlobalId, Core.Target.NpcId, location);
                    Clipboard.SetText(output);
                }
            }
        }

        private string pickupQuest = @"<!-- {0} -->
<If Condition=""IsQuestAcceptQualified({1})"">
    <PickupQuest NpcId=""{2}"" QuestId=""{1}"" XYZ=""{3}""/>
</If>";

        #endregion PickUpQuest

        #region TurnIn

        private void button3_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var questData = comboBox1.SelectedItem as QuestWork;
                if (questData != null)
                {
                    var location = Core.Target.Location.ToString().Remove(0, 1);
                    location = location.Remove(location.Length - 1, 1);
                    if (lbKeyItems.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbKeyItems.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var output = string.Format(turninQuestitems, questData.Name, questData.GlobalId, Core.Target.NpcId, location, itemids);
                        Clipboard.SetText(output);
                    }
                    else
                    if (lbBag1.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbBag1.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var output = string.Format(turninQuestitems, questData.Name, questData.GlobalId, Core.Target.NpcId, location, itemids);
                        Clipboard.SetText(output);
                    }
                    else
                    if (lbBag2.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbBag2.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var output = string.Format(turninQuestitems, questData.Name, questData.GlobalId, Core.Target.NpcId, location, itemids);
                        Clipboard.SetText(output);
                    }
                    else
                    if (lbBag3.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbBag3.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var output = string.Format(turninQuestitems, questData.Name, questData.GlobalId, Core.Target.NpcId, location, itemids);
                        Clipboard.SetText(output);
                    }
                    else
                    if (lbBag4.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbBag4.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var output = string.Format(turninQuestitems, questData.Name, questData.GlobalId, Core.Target.NpcId, location, itemids);
                        Clipboard.SetText(output);
                    }
                    else
                    {
                        var output = string.Format(turninQuest, questData.Name, questData.GlobalId, Core.Target.NpcId, location);
                        Clipboard.SetText(output);
                    }
                }
            }
        }

        private string turninQuest = @"<!-- {0} -->
<If Condition=""GetQuestStep({1}) == 255"">
    <TurnIn QuestId=""{1}"" NpcId=""{2}"" XYZ=""{3}""/>
</If>";

        private string turninQuestitems = @"<!-- {0} -->
<If Condition=""GetQuestStep({1}) == 255"">
    <TurnIn QuestId=""{1}"" NpcId=""{2}"" ItemIds=""{4}"" XYZ=""{3}""/>
</If>";

        #endregion TurnIn

        #region If

        private void button4_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                if (propertyGrid1.SelectedGridItem != null)
                {
                    var property = propertyGrid1.SelectedGridItem.Label;
                    if (property == "Step")
                    {
                        var questData = comboBox1.SelectedItem as QuestWork;
                        if (questData != null)
                        {
                            var output = string.Format(ifQuestStep, questData.Name, questData.GlobalId, questData.Step);
                            Clipboard.SetText(output);
                        }
                    }
                    else
                    {
                        var questData = comboBox1.SelectedItem as QuestWork;
                        if (questData != null)
                        {
                            var output = string.Format(ifQuestData, questData.Name, questData.GlobalId, property, propertyGrid1.SelectedGridItem.Value);
                            Clipboard.SetText(output);
                        }
                    }
                }
            }
        }

        private string ifQuestStep = @"<If Condition=""GetQuestStep({1}) == {2}"">

</If>";

        private string ifQuestData = @"<If Condition=""HasQuest({1}) and GetQuestById({1}).{2} == {3}"">
        
</If>";

        #endregion If

        #region TalkTo

        private void button5_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var questData = comboBox1.SelectedItem as QuestWork;
                if (questData != null)
                {
                    var location = Core.Target.Location.ToString().Remove(0, 1);
                    location = location.Remove(location.Length - 1, 1);
                    var output = string.Format(talkNothing, Core.Target.NpcId, location);
                    Clipboard.SetText(output);
                }
            }
        }

        private string talkNothing = @"<TalkTo InteractDistance=""3.0"" NpcId=""{0}"" XYZ=""{1}""/>";

        private string talkStandard = @"<TalkTo InteractDistance=""3.0"" QuestId=""66873"" StepId=""1"" NpcId=""1005932"" XYZ=""-1.236023, 33.38838, -244.1902""/>";

        #endregion TalkTo

        #region UseTransport

        private void button6_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var location = Core.Target.Location.ToString().Remove(0, 1);
                location = location.Remove(location.Length - 1, 1);
                var output = string.Format(UseTransport, Core.Target.NpcId, location);
                Clipboard.SetText(output);
            }
        }

        private string UseTransport = @"<UseTransport NpcId=""{0}"" InteractDistance=""3.5"" XYZ=""{1}""/>";

        #endregion UseTransport

        #region HandOver

        private void button7_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var questData = comboBox1.SelectedItem as QuestWork;
                if (questData != null)
                {
                    if (lbKeyItems.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbKeyItems.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var location = Core.Target.Location.ToString().Remove(0, 1);
                        location = location.Remove(location.Length - 1, 1);
                        var output = string.Format(Handover, itemids, questData.GlobalId, questData.Step, Core.Target.NpcId, location);
                        Clipboard.SetText(output);
                    }
                    if (lbBag1.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbBag1.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var location = Core.Target.Location.ToString().Remove(0, 1);
                        location = location.Remove(location.Length - 1, 1);
                        var output = string.Format(Handover, itemids, questData.GlobalId, questData.Step, Core.Target.NpcId, location);
                        Clipboard.SetText(output);
                    }
                    if (lbBag2.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbBag2.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var location = Core.Target.Location.ToString().Remove(0, 1);
                        location = location.Remove(location.Length - 1, 1);
                        var output = string.Format(Handover, itemids, questData.GlobalId, questData.Step, Core.Target.NpcId, location);
                        Clipboard.SetText(output);
                    }
                    if (lbBag3.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbBag3.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var location = Core.Target.Location.ToString().Remove(0, 1);
                        location = location.Remove(location.Length - 1, 1);
                        var output = string.Format(Handover, itemids, questData.GlobalId, questData.Step, Core.Target.NpcId, location);
                        Clipboard.SetText(output);
                    }
                    if (lbBag4.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbBag4.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var location = Core.Target.Location.ToString().Remove(0, 1);
                        location = location.Remove(location.Length - 1, 1);
                        var output = string.Format(Handover, itemids, questData.GlobalId, questData.Step, Core.Target.NpcId, location);
                        Clipboard.SetText(output);
                    }

                }
            }
        }

        private string Handover = @"<HandOver ItemIds=""{0}"" QuestId=""{1}"" StepId=""{2}"" NpcId=""{3}"" XYZ=""{4}""/>";

        #endregion HandOver

        #region MoveTo

        private void button8_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var name = Core.Target.Name.ToString();
                var location = Core.Target.Location.ToString().Remove(0, 1);
                location = location.Remove(location.Length - 1, 1);
                var output = String.Format(NoCombatMoveTo, name, location);
                Clipboard.SetText(output);
            }
        }

        private string NoCombatMoveTo = @"<NoCombatMoveTo Name=""{0}"" XYZ=""{1}""/>";

        #endregion MoveTo

        #region TeleportTo

        private void button9_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var target = Core.Target.NpcId;
                var output = string.Format(tpTo, target, WorldManager.GetZoneForAetheryteId(target));
                Clipboard.SetText(output);
            }
        }
  
        private string tpTo = @"<If Condition=""not IsOnMap({1})"">
    <TeleportTo AetheryteId=""{0}"" Name=""{1}""/>
</If>";

        #endregion TeleportTo

        #region UseObject

        private void button10_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var questData = comboBox1.SelectedItem as QuestWork;
                var objectId = Core.Target.NpcId;
                var location = Core.Target.Location.ToString().Remove(0, 1);
                location = location.Remove(location.Length - 1, 1);
                var output = string.Format(useObject, objectId, questData.GlobalId, questData.Step, location);
                Clipboard.SetText(output);

            }
        }

        private string useObject = @"<UseObject NpcId=""{0}"" QuestId=""{1}"" StepId=""{2}"" XYZ=""{3}"" Radius=""5""/>";

        #endregion UseObject

        #region UseItem

        private void button11_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var questData = comboBox1.SelectedItem as QuestWork;
                if (questData != null)
                {
                    if (lbKeyItems.SelectedItems.Count > 0)
                    {
                        var itemids = string.Join(",", lbKeyItems.SelectedItems.Cast<BagSlot>().Select(r => r.TrueItemId));
                        var location = Core.Target.Location.ToString().Remove(0, 1);
                        location = location.Remove(location.Length - 1, 1);
                        var output = string.Format(useItem, itemids, questData.GlobalId, questData.Step, Core.Target.NpcId, location);
                        Clipboard.SetText(output);
                    }
                }
            }
        }

        private string useItem = @"<UseItem NpcId=""{3}"" ItemId=""{0}"" QuestId=""{1}"" StepId=""{2}"" PostCombatDelay=""2"" XYZ=""{4}"" Radius=""5""/>";

        #endregion UseItem

        #region PickupDailyQuest

        private void button12_Click(object sender, EventArgs e)
        {
            using (Core.Memory.TemporaryCacheState(false))
            {
                var questData = comboBox1.SelectedItem as QuestWork;
                var location = Core.Target.Location.ToString().Remove(0, 1);
                location = location.Remove(location.Length - 1, 1);
                if (questData != null)
                {
                    var output = string.Format(pickupDailyQuest, questData.Name, questData.GlobalId, Core.Target.NpcId, location);
                    Clipboard.SetText(output);
                }
            }
        }

        private string pickupDailyQuest = @"<!-- {0} -->
<If Condition=""IsQuestAcceptQualified({1})"">
    <PickupDailyQuest NpcId=""{2}"" QuestId=""{1}"" XYZ=""{3}""/>
</If>";

        #endregion PickupDailyQuest
    }
}