﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Clio.Utilities;
using Clio.XmlEngine;
using ff14bot.Behavior;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.NeoProfiles;
using ff14bot.RemoteWindows;
using TreeSharp;
using Action = TreeSharp.Action;

namespace ff14bot.NeoProfiles
{
    [XmlElement("Transport")]

    public class TransportTag : ProfileBehavior
    {
        private bool _done;
        [Clio.XmlEngine.XmlAttribute("XYZ")]
        public Vector3 XYZ
        {
            get { return Position; }
            set { Position = value; }
        }

        public Vector3 Position;

        [XmlAttribute("Name")]
        public string Name { get; set; }

        [XmlAttribute("NpcId")]
        public uint NpcId { get; set; }

        [DefaultValue(3)]
        [XmlAttribute("Distance")]
        public float Distance { get; set; }

        public override bool IsDone { get { return _done; } }

        public bool completed = false;
		
		private bool IsFinished
        {
            get
            {
                var unit = GameObjectManager.GetObjectByNPCId(NpcId);

                if (unit != null)
                {
                    return !unit.IsVisible;
                }
                return true;
            }
        }

        protected Composite Behavior()
        {
            return new PrioritySelector(
				new Decorator(ret => Talk.DialogOpen,
                    new Action(r =>
                    {
                        Talk.Next();
                    })
                ),
                new Decorator(ret => completed,
                    new Action(r => _done = true)
                    ),
                new Decorator(ret => Core.Me.Location.Distance(Position) <= Distance && IsFinished,
                    new Action(ret => _done = true)),
                new Decorator(ret => Core.Me.Location.Distance(Position) <= Distance,
                    new Action(r =>
                    {
                        Navigator.PlayerMover.MoveStop();
                        if (NpcId != null)
                        {
                            Actionmanager.InteractWith(GameObjectManager.GetObjectByNPCId(NpcId));
                            completed = true;
						}
                    })
				),
                CommonBehaviors.MoveAndStop(ret => Position, Distance, stopInRange: true, destinationName: Name),
                new ActionAlwaysSucceed()
                );
        }

        protected override void OnResetCachedDone()
        {
            _done = false;
        }
        
        private Composite _cache;

        protected override void OnStart()
        {
            _cache = Behavior();
            TreeHooks.Instance.InsertHook("TreeStart", 0, _cache);
        }

        protected override void OnDone()
        {
            TreeHooks.Instance.RemoveHook("TreeStart", _cache);
        }
    }
}